import React, { Fragment } from 'react'
import { pick } from 'lodash'
import { connect } from 'react-redux'
import { required } from 'xpub-validators'
import { compose, withProps } from 'recompose'
import styled, { css } from 'styled-components'
import { Icon, Checkbox, ValidatedField, th } from '@pubsweet/ui'
import { reduxForm, Field, change as changeForm } from 'redux-form'
import { FormItems } from 'pubsweet-components-faraday/src/components'

import { Spinner } from '../UIComponents'

import {
  authorSuccess,
  authorFailure,
  getAuthorFetching,
} from '../../redux/authors'

import { authorKeys, parseEditedAuthors } from './utils'

const { Row, Label, RowItem, TextField, DefaultText } = FormItems

const renderCheckbox = ({ input }) => (
  <Checkbox checked={input.value} type="checkbox" {...input} />
)

const AuthorEdit = ({
  id,
  index,
  email,
  isFetching,
  handleSubmit,
  isSubmitting,
  setAuthorEdit,
  parseAuthorType,
  isCorresponding,
}) => (
  <Root>
    <Header>
      <TitleContainer>
        <span>{parseAuthorType(isSubmitting, isCorresponding, index)}</span>
        {!isSubmitting && (
          <Fragment>
            <Field component={renderCheckbox} name="edit.isCorresponding" />
            <label>Corresponding</label>
          </Fragment>
        )}
      </TitleContainer>

      <ButtonsContainer>
        <ClickableIcon
          data-test="author-edit-close"
          onClick={setAuthorEdit(-1)}
        >
          <Icon size={3}>x-circle</Icon>
        </ClickableIcon>
        {!isFetching ? (
          <ClickableIcon data-test="author-edit-save" onClick={handleSubmit}>
            <Icon size={3}>check-circle</Icon>
          </ClickableIcon>
        ) : (
          <Spinner size={3} />
        )}
      </ButtonsContainer>
    </Header>

    <Row>
      <RowItem data-test="edit-author-firstname" vertical withRightMargin>
        <Label>First name*</Label>
        <ValidatedField
          component={TextField}
          name="edit.firstName"
          validate={[required]}
        />
      </RowItem>
      <RowItem data-test="edit-author-lastname" vertical>
        <Label>Last name*</Label>
        <ValidatedField
          component={TextField}
          name="edit.lastName"
          validate={[required]}
        />
      </RowItem>
    </Row>

    <Row>
      <RowItem vertical withRightMargin>
        <Label>Email</Label>
        <DefaultText>{email}</DefaultText>
      </RowItem>
      <RowItem data-test="edit-author-affiliation" vertical>
        <Label>Affiliation*</Label>
        <ValidatedField
          component={TextField}
          name="edit.affiliation"
          validate={[required]}
        />
      </RowItem>
    </Row>
  </Root>
)

export default compose(
  connect(
    state => ({
      isFetching: getAuthorFetching(state),
    }),
    { changeForm, authorSuccess, authorFailure },
  ),
  withProps(props => ({
    initialValues: {
      edit: pick(props, authorKeys),
    },
  })),
  reduxForm({
    form: 'edit',
    onSubmit: (
      { edit: newAuthor },
      dispatch,
      { authors, changeForm, setAuthorEdit, parentForm },
    ) => {
      const newAuthors = parseEditedAuthors(newAuthor, authors)
      changeForm(parentForm, 'authors', newAuthors)
      setAuthorEdit(-1)()
    },
  }),
)(AuthorEdit)

// #region styled-components
const defaultText = css`
  color: ${th('colorText')};
  font-size: ${th('fontSizeBaseSmall')};
  font-family: ${th('fontReading')};
`

// const Row = styled.div`
//   display: flex;
//   flex-direction: row;
//   margin: ${th('subGridUnit')} 0;
// `

const TitleContainer = styled.div`
  align-items: center;
  display: flex;
  flex-direction: row;

  > span {
    ${defaultText};
    font-size: ${th('fontSizeHeading6')};
    margin-right: ${th('subGridUnit')};
    font-weight: 600;
    text-align: left;
  }

  label {
    ${defaultText};
    text-align: left;
  }
`

const ButtonsContainer = styled.div`
  align-items: center;
  display: flex;
  flex-direction: row;
  justify-content: flex-end;
`

const ClickableIcon = styled.div`
  align-items: center;
  cursor: pointer;
  display: flex;
  flex: 1;
  flex-direction: column;
  justify-content: center;
  margin: 0 calc(${th('subGridUnit')} * 2);
`

const Header = styled.div`
  align-items: center;
  display: flex;
  flex-direction: row;
  justify-content: space-between;
`

const Root = styled.div`
  border: ${th('borderDefault')};
  display: flex;
  flex-direction: column;
  margin: ${th('subGridUnit')} 0;
  padding: ${th('subGridUnit')};
`
// #endregion
